@@ -1,4 +1,4 @@
-{-# LANGUAGE ScopedTypeVariables, CPP #-}
+{-# LANGUAGE ScopedTypeVariables, CPP, ConstraintKinds #-}
 {-# OPTIONS_GHC -fno-warn-duplicate-exports #-}
 
 -- | Extra functions for "Control.Exception".
@@ -6,6 +6,7 @@
 --   and functions to catch\/ignore exceptions, including monomorphic (no 'Exception' context) versions.
 module Control.Exception.Extra(
     module Control.Exception,
+    Partial,
     retry, retryBool,
     errorWithoutStackTrace,
     showException, stringException,
@@ -21,6 +22,7 @@ import Control.Exception
 import Control.Monad
 import Data.List.Extra
 import Data.Functor
+import Partial
 import Prelude
 
 
@@ -73,7 +75,7 @@ ignore = void . try_
 --   Note that while 'fail' in 'IO' raises an 'IOException', this function raises an 'ErrorCall' exception.
 --
 -- > try (errorIO "Hello") == return (Left (ErrorCall "Hello"))
-errorIO :: String -> IO a
+errorIO :: Partial => String -> IO a
 errorIO = throwIO . ErrorCall
 
 
