@@ -2,15 +2,16 @@
 
 -- | Extra functions for "Control.Monad".
 --   These functions provide looping, list operations and booleans.
--- If you need a wider selection of monad loops and list generalisations,
--- see <http://hackage.haskell.org/package/monad-loops monad-loops>.
+--   If you need a wider selection of monad loops and list generalisations,
+--   see <http://hackage.haskell.org/package/monad-loops monad-loops>.
 module Control.Monad.Extra(
     module Control.Monad,
     whenJust, whenJustM,
+    whenMaybe, whenMaybeM,
     unit,
     maybeM, eitherM,
     -- * Loops
-    loopM, whileM,
+    loop, loopM, whileM,
     -- * Lists
     partitionM, concatMapM, concatForM, mconcatMapM, mapMaybeM, findM, firstJustM,
     fold1M, fold1M_,
@@ -36,8 +37,26 @@ whenJust mg f = maybe (pure ()) f mg
 
 -- | Like 'whenJust', but where the test can be monadic.
 whenJustM :: Monad m => m (Maybe a) -> (a -> m ()) -> m ()
+-- Can't reuse whenMaybe on GHC 7.8 or lower because Monad does not imply Applicative
 whenJustM mg f = maybe (return ()) f =<< mg
 
+
+-- | Like 'when', but return either 'Nothing' if the predicate was 'False',
+--   of 'Just' with the result of the computation.
+--
+-- > whenMaybe True  (print 1) == fmap Just (print 1)
+-- > whenMaybe False (print 1) == return Nothing
+whenMaybe :: Applicative m => Bool -> m a -> m (Maybe a)
+whenMaybe b x = if b then Just <$> x else pure Nothing
+
+-- | Like 'whenMaybe', but where the test can be monadic.
+whenMaybeM :: Monad m => m Bool -> m a -> m (Maybe a)
+-- Can't reuse whenMaybe on GHC 7.8 or lower because Monad does not imply Applicative
+whenMaybeM mb x = do
+    b <- mb
+    if b then liftM Just x else return Nothing
+
+
 -- | The identity function which requires the inner argument to be @()@. Useful for functions
 --   with overloaded return types.
 --
@@ -107,6 +126,15 @@ mapMaybeM op = foldr f (return [])
 
 -- | A looping operation, where the predicate returns 'Left' as a seed for the next loop
 --   or 'Right' to abort the loop.
+--
+-- > loop (\x -> if x < 10 then Left $ x * 2 else Right $ show x) 1 == "16"
+loop :: (a -> Either a b) -> a -> b
+loop act x = case act x of
+    Left x -> loop act x
+    Right v -> v
+
+-- | A monadic version of 'loop', where the predicate returns 'Left' as a seed for the next loop
+--   or 'Right' to abort the loop.
 loopM :: Monad m => (a -> m (Either a b)) -> a -> m b
 loopM act x = do
     res <- act x
